//+------------------------------------------------------------------+
//|                                                          MFI.mq5 |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright   "2009, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property description "Money Flow Index"
#include <MovingAverages.mqh>
//---- indicator settings
#property indicator_separate_window
#property indicator_buffers    4
#property indicator_plots      2
#property indicator_type1      DRAW_LINE
#property indicator_color1     DodgerBlue
#property indicator_type2      DRAW_LINE
#property indicator_color2     Red
#property indicator_width1  1
#property indicator_width2  1
#property indicator_label1  "MFI"
#property indicator_label2  "DEMA"
//---- input parameters
input int                 InpMFIPeriod=14;            // MFI Period
input ENUM_APPLIED_VOLUME InpVolumeType=VOLUME_TICK;  // Volumes
input int                 InpPeriodEMA=14;               // DEMA period
input int                 InpShift=0;                    // Indicator's shift
//---- buffers
double                    ExtMFIBuffer[];
double                    DemaBuffer[];
double                    Ema[];
double                    EmaOfEma[];
//--- global variable
int                       ExtMFIPeriod;
//+------------------------------------------------------------------+
//| Money Flow Index initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
//--- check for input value
   if(InpMFIPeriod<=0)
     {
      ExtMFIPeriod=14;
      Print("Parameter InpMFIPeriod has wrong value. Indicator will use value ",ExtMFIPeriod);
     }
   else ExtMFIPeriod=InpMFIPeriod;
 
   SetIndexBuffer(0,ExtMFIBuffer,INDICATOR_DATA);
   IndicatorSetString(INDICATOR_SHORTNAME,"DEMAMFI("+string(ExtMFIPeriod)+","+string(InpPeriodEMA)+")");
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,ExtMFIPeriod);
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
   //-------
   SetIndexBuffer(1,DemaBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,Ema,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,EmaOfEma,INDICATOR_CALCULATIONS);
//--- sets first bar from what index will be drawn
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,2*InpPeriodEMA-2);
//--- sets indicator shift
   PlotIndexSetInteger(0,PLOT_SHIFT,InpShift);
//---- end of initialization function
  }
//+------------------------------------------------------------------+
//| Money Flow Index                                                 |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,const int prev_calculated,
                const datetime &Time[],
                const double &Open[],
                const double &High[],
                const double &Low[],
                const double &Close[],
                const long &TickVolume[],
                const long &Volume[],
                const int &Spread[])
  {
//--- variables of indicator
   int    CalcPosition;
  //---- insufficient data
   if(rates_total<ExtMFIPeriod||rates_total<2*InpPeriodEMA-2)
      return(0);
      
   int limit;
   if(prev_calculated==0)
      limit=0;
   else limit=prev_calculated-1;
//--- start working
   if(prev_calculated<ExtMFIPeriod)
      CalcPosition=ExtMFIPeriod;
   else
      CalcPosition=prev_calculated-1;
//--- calculate MFI by volume
   if(InpVolumeType==VOLUME_TICK)
      CalculateDEMAMFI(CalcPosition,rates_total,High,Low,Close,TickVolume);
   else
      CalculateDEMAMFI(CalcPosition,rates_total,High,Low,Close,Volume);
      //--- calculate EMA   
      ExponentialMAOnBuffer(rates_total,prev_calculated,0,InpPeriodEMA,ExtMFIBuffer,Ema);
//--- calculate EMA on EMA array
      ExponentialMAOnBuffer(rates_total,prev_calculated,InpPeriodEMA-1,InpPeriodEMA,Ema,EmaOfEma);
//--- calculate DEMA
   for(int i=limit;i<rates_total && !IsStopped();i++)
      DemaBuffer[i]=2*Ema[i]-EmaOfEma[i];
//--- OnCalculate done. Return new prev_calculated
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Calculate MFI by volume from argument                            |
//+------------------------------------------------------------------+
void CalculateDEMAMFI(const int nPosition,
                     const int nRatesCount,
                     const double &HiBuffer[],
                     const double &LoBuffer[],
                     const double &ClBuffer[],
                     const long &VolBuffer[])
  {
  
   for(int i=nPosition;i<nRatesCount && !IsStopped();i++)
     {
      double dPositiveMF=0.0;
      double dNegativeMF=0.0;
      double dCurrentTP=TypicalPrice(HiBuffer[i],LoBuffer[i],ClBuffer[i]);
      for(int j=1;j<=ExtMFIPeriod;j++)
        {
         int    index=i-j;
         double dPreviousTP=TypicalPrice(HiBuffer[index],LoBuffer[index],ClBuffer[index]);
         if(dCurrentTP>dPreviousTP) dPositiveMF+=VolBuffer[index+1]*dCurrentTP;
         if(dCurrentTP<dPreviousTP) dNegativeMF+=VolBuffer[index+1]*dCurrentTP;
         dCurrentTP=dPreviousTP;
        }
      if(dNegativeMF!=0.0) ExtMFIBuffer[i]=100.0-100.0/(1+dPositiveMF/dNegativeMF);
      else                 ExtMFIBuffer[i]=100.0;
     }
  }
//+------------------------------------------------------------------+
//| Calculate typical price                                          |
//+------------------------------------------------------------------+
double TypicalPrice(const double dHi,const double dLo,const double dCl)
  {
   return (dHi+dLo+dCl)/3;
  }
//+------------------------------------------------------------------+



